package com.catarc.traceability.controller;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.catarc.traceability.entity.User;
import com.catarc.traceability.service.LoginService;
import com.catarc.traceability.util.Base64Decode;
import com.catarc.traceability.util.PasswordUtils;
import com.google.code.kaptcha.Constants;
import com.isoft91.web.base.controller.FileController;
import com.isoft91.web.base.response.AjaxResponse;

/**
 * 用户登录
 * @ProjectName:traceability
 * @ClassName:LoginController.java
 * @Description: 请务必自行补充代码核心描述
 * @Copyright: Copyright (c) 2017
 * @author: wangyao
 * @email: 842261834@qq.com 
 * @date 2017年9月4日 上午10:14:33
 * @version V1.0
 */
@Controller
@RequestMapping("/login")
public class LoginController extends FileController{
	
	@Autowired
	private LoginService loginService;
	
	
	
	
	/**
	 * 用户登录
	 * @MethodName:userLogin
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月28日 下午3:03:49
	 * @version V1.0
	 * @param userId
	 * @param pwd
	 * @return
	 * @throws UnknownHostException 
	 */
	@RequestMapping(value="/login" , method = RequestMethod.POST)
    @ResponseBody
    public AjaxResponse userLogin(User user,HttpServletRequest request, HttpServletResponse response) throws UnknownHostException {
		HttpSession session = request.getSession();  
	    String code = (String)session.getAttribute(Constants.KAPTCHA_SESSION_KEY);  
		String decodeUserName = new String(Base64Decode.decode(user.getUserName()));
		String decodePwd = new String (Base64Decode.decode(user.getUserPwd()));
		AjaxResponse ajaxResponse = new AjaxResponse();
		User delUser = loginService.selectLoginTimes(decodeUserName);
		String adminRoleId="";
    	if(null == delUser){
    		ajaxResponse.setMessage("该用户不存在");
    		return ajaxResponse;
    	}else{
    		adminRoleId = loginService.selectRoleId(delUser.getId());
    	}
		User loginTimes = loginService.selectLoginTimes(decodeUserName);
		if(loginTimes.getLoginTimes() > 4){
			//用户状态置为停用
			loginService.updateUserStateWhileLock(decodeUserName);
			ajaxResponse.setMessage("您的账号已锁，请联系管理员");
			return ajaxResponse;
		}
        User selUser = new User();
        selUser.setUserName(decodeUserName);
        selUser.setUserPwd(PasswordUtils.passwordEncrypt(decodePwd));
        User res = loginService.getEntity(selUser);
        if(null != res){
        	if(res.getStatus().equals("0")){
        		ajaxResponse.setMessage("该用户已经被停用");
        		return ajaxResponse;
        	}
        	if(!user.getIdCode().toLowerCase().equals(code)){
        		ajaxResponse.setMessage("验证码不正确");
        		return ajaxResponse;
        	}else{
        	InetAddress address = InetAddress.getLocalHost();//获取的是本地的IP地址 //PC-20140317PXKX/192.168.0.121
        	String hostAddress = address.getHostAddress();//192.168.0.121    
        	String roleId = loginService.userRoleId(res.getId());
        	ajaxResponse.setMessage("success");
        	session.setAttribute("loginUserId", res.getId());
        	session.setAttribute("userRoleId", roleId);
        	session.setAttribute("userIpAddress", hostAddress);
        	session.setAttribute("loginUserName", res.getUserName());
        	session.setAttribute("password", res.getUserPwd());
        	user.setLoginTimes(0);
        	user.setLastLoginTime(new Date());    	
        	user.setUserName(decodeUserName);
        	loginService.updateLoginInfo(user);
        	
        	}
        }else{
        	//绕开admin用户
        	if(decodeUserName.equals("admin")||adminRoleId.equals("C2A38AF5FBD84D268F01E07A9E4C540E")){
        		if(delUser.getLoginTimes() >= 3){
        			//如果登录次数已经等于4，并且又进了这个方法
        			//用户状态置为停用
        			loginService.updateUserStateWhileLockTo(decodeUserName);
        		}
        	}else{
        		user.setLoginTimes(delUser.getLoginTimes()+1);
        		user.setUserName(decodeUserName);
        		loginService.updateLoginTimes(user);
        	}
        	ajaxResponse.setMessage("用户名或密码错误，请重新输入");
        }
        return ajaxResponse;
    }
	
	/**
	 * 用户登出
	 * @MethodName:logout
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月28日 下午3:04:29
	 * @version V1.0
	 * @return
	 */
	@ResponseBody
    @RequestMapping("/logout")
    public AjaxResponse logout(){
    	AjaxResponse ajaxResponse = new AjaxResponse();
		session.removeAttribute("loginUserId");
		session.removeAttribute("userRoleId");
    	session.invalidate();   
		ajaxResponse.setMessage("success");
		return ajaxResponse;
    }
	
	/**
	 * 查询用户权限
	 * @MethodName:selectUserAuthority
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月29日 下午8:31:41
	 * @version V1.0
	 * @return
	 */
	@RequestMapping("/selectUserAuthority")
	@ResponseBody
	public AjaxResponse selectUserAuthority(){
		String userId = (String) session.getAttribute("loginUserId");
		String roleId = (String) session.getAttribute("userRoleId");
		AjaxResponse result = new AjaxResponse();
		try {
			Map list = loginService.selectUserAuthority(roleId);

			Map<String, Object> map = new HashMap<String, Object>();
			map.put("result", list);
			result.setResponseData(map);
		} catch (Exception e) {
			result.setMessage("roleId为空，未登陆系统");
		}

		return result;
	}
	
	/**
	 * 用户修改密码
	 * @MethodName:updatePwd
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月31日 上午11:13:51
	 * @version V1.0
	 * @param user
	 * @return
	 */
	@RequestMapping("/updatePwd")
	@ResponseBody
	public AjaxResponse updatePwd(User user){
		AjaxResponse res = new AjaxResponse();
		String userId = (String) session.getAttribute("loginUserId");
		User pwdUser = new User();
		User pwdOrigin = new User();
		pwdOrigin.setId(userId);
		pwdOrigin.setUserPwd(PasswordUtils.passwordEncrypt(user.getOriginPwd()));
		pwdOrigin.setFirstpwd(PasswordUtils.passwordEncrypt(user.getFirstpwd()));
		int judge =loginService.originPwd(pwdOrigin);
		if(judge==1){
		pwdUser.setId(userId);
		pwdUser.setUpdateTime(new Date());
		pwdUser.setUserPwd(PasswordUtils.passwordEncrypt(user.getUserPwd()));
		pwdUser.setOutDate(new Date());
		int flag = loginService.updatePwd(pwdUser);
		if(flag > 0){
			
			res.setMessage("success");
		}else{
			res.setMessage("failure");
		}
		}else if(judge==2){
			res.setMessage("originPwdError");
		}else{
			res.setMessage("originPwd");
		}
			return res;
}
	/**
	 * 检查用户密码是否三个月未修改
	 * @MethodName:checkUserSecret
	 * @author: 李星汕
	 * @email: 2534347714@qq.com 
	 * @date 2017年9月26日 下午4:54:30
	 * @version V1.0
	 * @return
	 */
	@RequestMapping("/checkUserSecret")
	@ResponseBody
	public AjaxResponse checkUserSecret(User user,HttpServletRequest request, HttpServletResponse response) throws UnknownHostException {
		String username =(String)session.getAttribute("loginUserName"); //获取登录用户名
		user.setUserName(username);
		AjaxResponse ajaxResponse = new AjaxResponse();
		User delUser = loginService.selectLoginTimes(username);  //查询信息
		Calendar c = Calendar.getInstance(); 
    	c.setTime(delUser.getOutDate()); //获取当前用户修改密码时间  
    	c.add(Calendar.DAY_OF_MONTH, 90); //设置检查时间为修改密码的90天后
    	int dateTag = new Date().compareTo(c.getTime()); //超过90天返回1  否则返回-1
    	if(dateTag == 1){
    		ajaxResponse.setMessage("outdate"); 
    		return ajaxResponse;
    	}else{
    		ajaxResponse.setMessage("valid");
    		return ajaxResponse;
    	}
	}
}